@@ -3,7 +3,7 @@ package Thread::Suspend;
 # Make sure we have version info for this module
 # Make sure we do everything by the book from now on
 
-$VERSION = '0.02';
+$VERSION = '0.03';
 use strict;
 
 # Only load the things that are necessary
@@ -20,6 +20,10 @@ use threads::shared ();
 
 our $running =
  defined( $Thread::Running::VERSION ) ? \&Thread::Running::running  : '';
+
+# Initialize flag to indicate we can kill threads
+
+our $cankill; BEGIN { $cankill = defined $Thread::Exit::VERSION };
     
 # Hash with suspended thread ID's
 
@@ -50,7 +54,7 @@ Thread::Signal->automatic( $signal );
 #---------------------------------------------------------------------------
 #  IN: 1 class (ignored) or object to be checked
 #      2..N additional thread (ID's) that should be checked (default: all)
-# OUT: 1..N thread ID's that are still running
+# OUT: 1..N thread ID's that have been suspended
 
 sub threads::suspend {
 
@@ -64,7 +68,7 @@ sub threads::suspend {
 #---------------------------------------------------------------------------
 #  IN: 1 class (ignored) or object to be checked
 #      2..N additional thread (ID's) that should be checked (default: all)
-# OUT: 1..N thread ID's that can be join()ed
+# OUT: 1..N thread ID's that have been resumed
 
 sub threads::resume {
 
@@ -75,6 +79,20 @@ sub threads::resume {
     goto &resume;
 } #threads::resume
 
+#---------------------------------------------------------------------------
+#  IN: 1 class (ignored) or object to be checked
+#      2..N additional thread (ID's) that should be checked (default: all)
+# OUT: 1..N thread ID's that are suspended
+
+sub threads::suspended {
+
+# Lose the class
+# Go do the actual check
+
+    shift unless ref $_[0];
+    goto &suspended;
+} #threads::suspended
+
 #---------------------------------------------------------------------------
 #  IN: 1 class/object (ignored)
 
@@ -85,6 +103,20 @@ sub threads::iambusy { goto &iambusy } #threads::iambusy
 
 sub threads::iamdone { goto &iamdone } #threads::iamdone
 
+#---------------------------------------------------------------------------
+#  IN: 1 class (ignored) or object to be checked
+#      2..N additional thread (ID's) that should be checked (default: all)
+# OUT: 1..N thread ID's that were killed
+
+sub threads::kill {
+
+# Lose the class
+# Go do the actual check
+
+    shift unless ref $_[0];
+    goto &kill;
+} #threads::kill
+
 #---------------------------------------------------------------------------
 # NOTE: signal handling subroutine _must_ exist and cannot be AUTOLOADed
 
@@ -95,14 +127,18 @@ sub _suspending {
     return if $busy;
 
 # Obtain the thread ID
-# While we're supposed to be suspended
+# While we're supposed to be suspended or killed
+#  Exit now if we're being killed
 #  Wait until we can get a lock
+#  Mark this thread as actually suspended
 #  Release the lock, wait until someone tells us to check again
 
     my $tid = threads->tid;
     while ($suspended[$tid]) {
+        exit( undef ) if $suspended[$tid] == -1;
         {
          lock @suspended;
+         $suspended[$tid] = 1;
          threads::shared::cond_wait( @suspended );
         }
     }
@@ -135,7 +171,7 @@ sub suspend {
 
 #  For all of the threads specified
 #   Make sure we have a thread ID
-#   Reloop if already suspended
+#   Reloop if already suspended or killed
 #   Mark this thread as suspended
 #   Add this thread to the list
 
@@ -177,7 +213,7 @@ sub resume {
 
         foreach (@_) {
             my $tid = ref( $_ ) ? $_->tid : $_;
-            next unless $suspended[$tid];
+            next if $suspended[$tid] <= 0;
             $suspended[$tid] = 0;
             push @tid,$tid;
         }
@@ -190,6 +226,32 @@ sub resume {
     return wantarray ? @tid : @tid == @_;
 } #resume
 
+#---------------------------------------------------------------------------
+#  IN: 1..N thread (ID's) that should be checked (default: all)
+# OUT: 1..N thread ID's that have been suspended
+
+sub suspended {
+
+# Initialize the thread ID's
+# Set default thread ID's to handle if none specified
+#  Make sure we're the only ones accessing
+#  For all of the threads specified
+#   Make sure we have a thread ID
+#   Add this thread to the list if marked as suspended
+# Return list of thread ID's or whether all have been killed
+
+    my @tid;
+    @_ = _running() unless @_;
+    {
+        lock @suspended;
+        foreach (@_) {
+            my $tid = ref( $_ ) ? $_->tid : $_;
+            push( @tid,$tid ) if $suspended[$tid] == 1;
+        }
+    }
+    return wantarray ? @tid : @tid == @_;
+} #suspended
+
 #---------------------------------------------------------------------------
 #  IN: 1 class/object (ignored)
 
@@ -200,6 +262,47 @@ sub iambusy { $busy++ } #iambusy
 
 sub iamdone { _suspending() if $busy and !--$busy } #iamdone
 
+#---------------------------------------------------------------------------
+#  IN: 1..N thread (ID's) that should be checked (default: all)
+# OUT: 1..N thread ID's that have been killed
+
+sub kill {
+
+# Die now if we can't kill threads
+
+    die "Cannot kill other threads because Thread::Exit was not loaded\n"
+     unless $cankill;
+
+# Initialize the thread ID's
+# Set default thread ID's to handle if none specified
+#  Make sure we're the only ones accessing
+
+    my @tid;
+    @_ = _running() unless @_;
+    {
+        lock @suspended;
+
+#  For all of the threads specified
+#   Make sure we have a thread ID
+#   Reloop if already killed
+#   Mark this thread as killed
+#   Add this thread to the list
+
+        foreach (@_) {
+            my $tid = ref( $_ ) ? $_->tid : $_;
+            next if $suspended[$tid] == -1;
+            $suspended[$tid] = -1;
+            push @tid,$tid;
+        }
+    }
+
+# Signal the threads
+# Return list of thread ID's or whether all have been killed
+
+    Thread::Signal->signal( $signal, @tid );
+    return wantarray ? @tid : @tid == @_;
+} #kill
+
 #---------------------------------------------------------------------------
 
 # Methods needed by Perl
@@ -218,7 +321,7 @@ sub import {
 
     shift;
     my $namespace = (scalar caller() ).'::';
-    @_ = qw(suspend resume iambusy iamdone) unless @_;
+    @_ = qw(suspend resume suspended iambusy iamdone kill) unless @_;
     no strict 'refs';
     *{$namespace.$_} = \&$_ foreach @_;
 } #import
@@ -230,7 +333,14 @@ sub import {
 #---------------------------------------------------------------------------
 # OUT: 1..N all the thread ID's currently running (including detached threads)
 
-sub _running { $running ? &$running : () } #_running
+sub _running {
+
+# Go fetch the thread ID's of the running threads if we can
+# Die now if we can't find out which threads are running
+
+    goto &$running if $running; 
+    die "Cannot find out which other threads are running because Thread::Running was not loaded\n";
+} #_running
 
 #---------------------------------------------------------------------------
 # OUT: 1..N all the thread ID's currently suspended (including detached threads)
@@ -254,7 +364,7 @@ Thread::Suspend - suspend and resume threads from another thread
 
 =head1 SYNOPSIS
 
-    use Thread::Suspend;             # exports suspend, resume, iambusy, iamdone
+    use Thread::Suspend;             # exports all methods
     use Thread::Suspend qw(suspend); # only exports suspend()
     use Thread::Suspend ();          # threads class methods only
 
@@ -264,12 +374,21 @@ Thread::Suspend - suspend and resume threads from another thread
     threads->suspend( $tid );        # suspend by thread ID
     threads->suspend;                # suspend all (other) threads
 
+    $thread->suspended;              # true if thread is suspended
+    threads->suspended;              # true if all threads suspended
+
     $thread->resume;                 # resume a single thread
     threads->resume;                 # resume all suspended threads
 
     threads->iambusy;                # don't allow suspending in thread
     threads->iamdone;                # allow suspending again in thread
 
+    use Thread::Exit();
+    use Thread::Suspend();
+    $thread->kill;                   # kill thread by object
+    threads->kill;                   # kill all (other) threads
+    threads->kill( @thread );        # kill only listed threads
+
 =head1 DESCRIPTION
 
                   *** A note of CAUTION ***
@@ -281,9 +400,9 @@ Thread::Suspend - suspend and resume threads from another thread
 
                   *************************
 
-This module adds one feauture to threads that are sorely missed by some:
-the capability to suspend execution of a thread and be able to resume it
-when needed afterwards.
+This module adds the following features to threads that are sorely missed by
+some: the capability to suspend, to resume and to terminate execution of a
+thread.
 
 =head1 METHODS
 
@@ -304,22 +423,18 @@ obtained by the C<threads::tid()> method).
 If called as a class method or as a subroutine without parameters, then it
 will suspend all running threads but the current thread.  This can only
 be done if L<Thread::Running> has been loaded prior to loading Thread::Suspend.
+If this was not done, then calling "suspend" as a class method without
+parameters will B<die>.
 
 If called as an instance method without parameters, it will only check the
 thread associated with the object.
 
-If called in void context, then no check will be performed whether the
-specified threads are actually suspended.  If called in scalar or list
-context, the call will block until all specified threads have indicated
-back that they've been suspended.  Due to latency, this may take a little
-while.  By default a 5 second timeout will be applied: if the threads still
-haven't indicated that they're suspended by then, then execution of the
-current thread will continue.
+In list context it returns the thread ID's of the threads that have been to
+to suspend.  In scalar context, it returns 1 or 0 to indicate whether all of
+the specified threads have been told to suspend.
 
-In list context it returns the thread ID's of the threads that have indicated
-that they have been suspended.  In scalar context, it returns 1 or 0 to
-indicate whether all of the specified threads have indicated that they have
-been suspended.
+You can call the L<suspended> method to find out which threads are actually
+suspended.
 
 =head2 resume
 
@@ -337,17 +452,31 @@ If called as a class method or as a subroutine without parameters, then it
 will check all threads of which it knows.  If called as an instance method
 without parameters, it will only check the thread associated with the object.
 
-If called in void context, then no check will be performed whether the
-specified threads are actually resumed.  If called in scalar or list
-context, the call will block until all specified threads have indicated
-back that they've been resumed.  Due to latency, this may take a little
-while.  By default a 5 second timeout will be applied: if the threads still
-haven't indicated that they've resumed by then, then execution of the
-current thread will continue.
+In list context it returns thread id's of the threads that have been told
+to resume execution.  In scalar context, it just returns 1 or 0 to indicate
+whether all of the (implicitely) indicated threads were told to resume
+execution.
 
-In list context it returns thread id's of the threads that have resumed.
-In scalar context, it just returns 1 or 0 to indicate whether all of the
-(implicitely) indicated threads have resumed.
+=head2 suspended
+
+ $thread->suspended;                # true if thread is suspended
+ threads->suspended( $thread );     # same
+ threads->suspended( $thread->tid); # same
+
+ threads->suspended;                # true if all other threads suspended
+ threads->suspended( @tid );        # true if given threads suspended
+
+The "suspended" method allows you to check whether one or more threads are
+actually suspended.  It accepts one or more thread objects or thread ID's
+(as obtained by the C<threads::tid()> method).
+
+If called as a class method or as a subroutine without parameters, then it
+will check all threads of which it knows.  If called as an instance method
+without parameters, it will only check the thread associated with the object.
+
+In list context it returns thread id's of the threads that are actually
+suspended.  In scalar context, it just returns 1 or 0 to indicate whether
+all of the (implicitely) indicated threads are suspended.
 
 =head2 iambusy
 
@@ -365,7 +494,7 @@ to "iamdone" is suspending of the current thread allowed again.
 
 Please note that if you call the "iambusy" method at the beginning of the
 execution of a thread, and never call the "iamdone" method in that thread,
-then that thread  will B<never> get suspended.
+then that thread  will B<never> get suspended, but it may get killed.
 
 =head2 iamdone
 
@@ -382,6 +511,44 @@ If suspending of the thread is allowed again, then immediate suspending of
 the thread will occur if another thread asked for suspending of the current
 thread while the current thread was busy.
 
+=head2 kill
+
+ $thread->kill;            # kill only this thread
+ kill( $thread );          # same
+
+ threads->kill( @tid );    # kill all threads indicated by thread ID's
+ threads->kill;            # kill all running threads
+
+                  *** A note of CAUTION ***
+
+ The C<kill()> method/subroutine can only work if the Thread::Exit module
+ has been loaded B<before> the Thread::Suspend module is loaded.
+ Therefore, calling the C<kill()> method/subroutine will B<die> when
+ called if Thread::Exit had not been loaded before Thread::Suspend was
+ loaded.
+
+                  *************************
+
+The "kill" method allows you to stop execution of one or more threads.
+It accepts one or more thread objects or thread ID's (as obtained by the
+C<threads::tid()> method).
+
+If called as a class method or as a subroutine without parameters, then it
+will kill all running threads but the current thread.  This can only
+be done if L<Thread::Running> has been loaded prior to loading Thread::Suspend.
+If this was not done, then calling "kill" as a class method without parameters
+will B<die>.
+
+If called as an instance method without parameters, it will only check the
+thread associated with the object.
+
+In list context it returns the thread ID's of the threads that have been
+attempted to be killed.  In scalar context, it returns 1 or 0 to indicate
+whether all of the specified threads have been attempted to be killed.
+
+You can call Thread::Running's C<running> method to find out which threads
+are actually running.
+
 =head1 CAVEATS
 
 This module is dependent on the L<Thread::Signal> module, with all of its
@@ -394,6 +561,13 @@ having received a signal.
 This module uses the L<load> module to make sure that subroutines are loaded
 only when they are needed.
 
+The dependencies on Thread::Running and Thread::Exit are made conditional in
+such a way that they will not use any more memory (which is B<very> important
+when using threads) then necessary.  Unfortunately, these modules can not be
+loaded on demand, as they alter subtle characteristics of the environment at
+compile time, on which Thread::Suspend must be able to depend to act as
+expected.  So there is no easy way around this seemingly awkward situation.
+
 =head1 WHY NOT USE "SIGSTOP" AND "SIGCONT"?
 
 Many operating systems already support the signals SIGSTOP (for halting
@@ -421,6 +595,6 @@ modify it under the same terms as Perl itself.
 
 =head1 SEE ALSO
 
-L<threads>, L<Thread::Signal>, L<Thread::Running>, L<load>.
+L<threads>, L<Thread::Signal>, L<Thread::Running>, L<Thread::Exit>, L<load>.
 
 =cut
